package services.pescado.fusionservice.impl;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TimeZone;
import java.util.logging.Logger;

import services.pescado.common.PESCaDOException;
import services.pescado.common.PESCaDOExceptionEx;
import services.pescado.common.ParameterType;
import Helper.DateAccessor;
import de.fraunhofer.iosb.sosclient.GMLHelper;


/**
 * Encapsulates parameters from a service call.
 * 
 * @author J&uuml;rgen Mo&szlig;graber
 * @author Desiree Hilbring
 */
public class Parameters
{
	/** The namespace of the PESCaDO project. */
	public final static String							NS_PESCADO								= "http://www.pescado-project.eu/ontology/pescado.owl#";

	/** The user. */
	public final static String							USER_PARAM								= "user";
	/** The activity. */
	public final static String							ACTIVITY_PARAM						= "activity";
	/** The time start. */
	public static final String							TIME_START_PARAM					= "timeStart";
	/** The time end. */
	public static final String							TIME_END_PARAM						= "timeEnd";
	/** The geo type. */
	public static final String							GEO_TYPE_PARAM						= "geoType";
	/** The geo polygon. */
	public static final String							GEO_POLYGON_PARAM					= "geoPolygon";

	public static final String							FREQUENCY									= "frequency";

	public static final String							ONLY_IF_EXCEEDANCE				= "onlyIfExceedance";

	public static final String							DAILY_TIME								= "dailyTime";

	public static final String							HOURS_IN_PAST							= "hoursInPastToConsider";

	public static final String							HOURS_IN_FUTURE						= "hoursInFutureToConsider";

	public static final String							SUBMISSION_TIME						= "hasRequestSubmissionDateTime";

	public static final String							USE_CASE									= "NotDetailedAirQuality";

	private final static String							FORECASTED								= "http://www.pescado-project.eu/ontology/pescado.owl#forecasted";
	private final static String							OBSERVED									= "http://www.pescado-project.eu/ontology/pescado.owl#observed";

	/** Parameter from time. */
	public static final String							PARAM_FROM_DATE_TIME			= NS_PESCADO + "hasFromDateTime";
	/** Parameter end time. */
	public static final String							PARAM_TO_DATE_TIME				= NS_PESCADO + "hasToDateTime";
	/** Parameter geo area type. */
	public static final String							PARAM_GEO_AREA_TYPE				= NS_PESCADO + "hasGeoAreaType";
	/** Parameter geo polygon. */
	public static final String							PARAM_GEO_AREA_LOCATION		= NS_PESCADO + "hasGeoAreaLocation";

	public static final String							PARAM_ONLY_IF_EXCEEDANCE	= NS_PESCADO + "hasTaskOnlyIfExceendance";

	public static final String							PARAM_FREQUENCY						= NS_PESCADO + "hasTaskFrequency";

	public static final String							PARAM_DAILY_TIME					= NS_PESCADO + "hasTaskDailyAtTime";

	public static final String							PARAM_HOURSE_IN_PAST			= NS_PESCADO + "hasTaskPastHours";

	public static final String							PARAM_HOURSE_IN_FUTURE		= NS_PESCADO + "hasTaskFutureHours";

	private static final Set<String>				IGNORE_PARAMS							= new HashSet<String> (2);
	static
	{
		Collections.addAll (IGNORE_PARAMS, USER_PARAM, ACTIVITY_PARAM, TIME_START_PARAM, TIME_END_PARAM, GEO_TYPE_PARAM, GEO_POLYGON_PARAM);
	}

	// private final static Pattern DATETIMEPATTERN = Pattern.compile
	// ("([0-9]{2}):([0-9]{2}):([0-9]{2})([.]([0-9]{1,3}))?((Z)|(([+]|-)([0-9]{2}):([0-9]{2})))?");

	private final Map<String, List<Object>>	ps												= new HashMap<String, List<Object>> ();
	private final boolean										isArea;
	private String													timeStart;
	private String													timeEnd;
	private final List<Double>							polygon;
	private final boolean										onlyIfExceedance;
	private final String										frequency;
	private final String										dailyTime;
	private int															hoursInPast;
	private int															hoursInFuture;
	private final String										submissionTime;
	private String													useCase;

	private final static Logger							LOGGER										= Logger.getLogger ("UIS");


	/**
	 * Constructor.
	 * 
	 * @param parameters The parameters
	 * @throws ParseException Could not parse the values of the polygon
	 */
	public Parameters (final List<ParameterType> parameters) throws ParseException
	{
		for (final ParameterType p: parameters)
			this.ps.put (p.getProperty (), p.getContent ());

		this.useCase = USE_CASE;

		this.timeStart = this.getTextValue (TIME_START_PARAM);
		this.timeEnd = this.getTextValue (TIME_END_PARAM);

		final String geoType = this.getTextValue (GEO_TYPE_PARAM);
		this.isArea = geoType == null || "area".equals (geoType);

		final String p = this.getTextValue (GEO_POLYGON_PARAM);
		this.polygon = p != null ? GMLHelper.parseDoubleValues (p, " ", ",", ".") : null;

		String exd = this.getTextValue (ONLY_IF_EXCEEDANCE);
		if (exd != null)
			this.onlyIfExceedance = this.getTextValue (ONLY_IF_EXCEEDANCE).equals ("true") ? true : false;
		else
			this.onlyIfExceedance = false;

		this.frequency = this.getTextValue (FREQUENCY);
		this.dailyTime = this.getTextValue (DAILY_TIME);
		this.submissionTime = this.getTextValue (SUBMISSION_TIME);

		String hip = this.getTextValue (HOURS_IN_PAST);
		if (hip != null)
			this.hoursInPast = Integer.parseInt (this.getTextValue (HOURS_IN_PAST));
		else
			this.hoursInPast = 0;
		String hif = this.getTextValue (HOURS_IN_FUTURE);
		if (hif != null)
			this.hoursInFuture = Integer.parseInt (this.getTextValue (HOURS_IN_FUTURE));
		else
			this.hoursInFuture = 0;

		// Check if the start and end of the polygon are identical
		if (this.polygon == null || this.polygon.isEmpty ())
			return;
		final int size = this.polygon.size ();
		if (size < 2)
			return;
		final Double x1 = this.polygon.get (0);
		final Double y1 = this.polygon.get (1);
		if (!x1.equals (this.polygon.get (size - 2)) || !y1.equals (this.polygon.get (size - 1)))
		{
			this.polygon.add (x1);
			this.polygon.add (y1);
		}

	}


	/**
	 * Return the submission time of the task.
	 * 
	 * @return submission time
	 */
	public String getSubmissionTime ()
	{
		return this.submissionTime;
	}


	/**
	 * Check if report shall only be sent if exceedances are contained.
	 * 
	 * @return exceedance check
	 */
	public boolean isOnlyIfExceedance ()
	{
		return this.onlyIfExceedance;
	}


	/**
	 * Get the frequency of sending a report.
	 * 
	 * @return frequency
	 */
	public String getFrequency ()
	{
		if (this.frequency.contains ("daily"))
			return NS_PESCADO + "dailyTaskFrequencyType";
		return NS_PESCADO + "hourlyTaskFrequencyType";
	}


	/**
	 * Get the time for the daily creation of the report.
	 * 
	 * @return time for report creation
	 */
	public String getDailyTime ()
	{
		return this.dailyTime;
	}


	/**
	 * Get the past hours to consider in report generation.
	 * 
	 * @return number of past hours.
	 */
	public int getHoursInPast ()
	{
		return this.hoursInPast;
	}


	/**
	 * Get the number of future hours to consider in report generation.
	 * 
	 * @return number of future hours.
	 */
	public int getHoursInFuture ()
	{
		return this.hoursInFuture;
	}


	/**
	 * Get the user who sent the request.
	 * 
	 * @return The user, an URI of the ontology
	 */
	public String getUser ()
	{
		return this.getTextValue (USER_PARAM);
	}


	/**
	 * Get the activity.
	 * 
	 * @return The activity, an URI of the ontology
	 */
	public String getActivity ()
	{
		return this.getTextValue (ACTIVITY_PARAM);
	}


	/**
	 * Get the start date/time.
	 * 
	 * @return ISO encoded
	 * @throws PESCaDOException
	 */
	public String getStartDateTime () throws PESCaDOException
	{
		LOGGER.info ("timeStart: " + this.timeStart);
		// if (this.timeStart == null)
		// {
		// valid for Scheduler
		if (this.frequency != null)
		{
			if (this.frequency.equals ("daily"))
			{
				if (this.dailyTime != null)
				{
					try
					{
						Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
						c.add (Calendar.HOUR_OF_DAY, -this.hoursInPast);
						// LOGGER.info ("ph " + this.hoursInPast);
						if (this.hoursInPast == 24)
						{
							c.set (Calendar.HOUR_OF_DAY, 0);
							c.set (Calendar.MINUTE, 0);
							c.set (Calendar.SECOND, 0);
							c.set (Calendar.MILLISECOND, 0);
						}
						// LOGGER.info ("start daily " +
						// DateAccessor.formatISODateTimeWithTimezone (c));
						// Underlying System works with local timezone!
						c.setTimeZone (TimeZone.getDefault ());
						// LOGGER.info ("start daily stz " +
						// DateAccessor.formatISODateTimeWithTimezone (c));
						return DateAccessor.formatISODateTimeWithTimezone (c);
					}
					catch (ParseException e)
					{
						e.printStackTrace ();
						throw PESCaDOExceptionEx.with ("Exception raised calculating start time", "", "FS", e);
					}
				}
			}
			// hourly
			else
			{
				Calendar c = Calendar.getInstance ();
				c.add (Calendar.HOUR_OF_DAY, -this.hoursInPast);
				if (this.hoursInPast == 24)
				{
					c.set (Calendar.HOUR_OF_DAY, 0);
					c.set (Calendar.MINUTE, 0);
					c.set (Calendar.SECOND, 0);
					c.set (Calendar.MILLISECOND, 0);
				}
				c.setTimeZone (TimeZone.getDefault ());
				// LOGGER.info ("start hourly " +
				// DateAccessor.formatISODateTimeWithTimezone (c));
				return DateAccessor.formatISODateTimeWithTimezone (c);
			}
		}
		// valid for normal request
		else
		{
			// HACK for Iphone
			if (this.timeStart.contains ("Z"))
			{
				LOGGER.info ("ts " + this.timeStart);
				String [] ta = this.timeStart.split ("Z");
				String newTimeFormat = ta[0] + ":00+00:00";
				LOGGER.info ("ntf " + newTimeFormat);
				return newTimeFormat;
			}
			return this.timeStart;
		}
		return this.timeStart;
	}


	/**
	 * Get the start date/time.
	 * 
	 * @return ISO encoded
	 * @throws PESCaDOException
	 */
	public String getStartDateTime (String timeType) throws PESCaDOException
	{
		LOGGER.info ("timetype: " + timeType);
		LOGGER.info ("timeStart: " + this.timeStart);
		LOGGER.info ("timeEnd: " + this.timeEnd);
		LOGGER.info ("frequency: " + this.frequency);
		LOGGER.info ("dailyTime: " + this.dailyTime);
		LOGGER.info ("ph: " + this.getHoursInPast ());
		LOGGER.info ("fh: " + this.getHoursInFuture ());
		LOGGER.info ("subtime: " + this.submissionTime);
		LOGGER.info ("use case: " + this.useCase);
		// if (this.timeStart == null)
		// {
		// valid for Scheduler
		if (this.frequency != null)
		{
			if (this.frequency.equals ("daily"))
			{
				if (this.dailyTime != null)
				{
					try
					{
						if (timeType.equals (OBSERVED))
						{
							LOGGER.info ("observed " + (-this.getHoursInPast ()));
							Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
							c.add (Calendar.HOUR_OF_DAY, -this.getHoursInPast ());
							if (this.hoursInPast == 24)
							{
								c.set (Calendar.HOUR_OF_DAY, 0);
								c.set (Calendar.MINUTE, 0);
								c.set (Calendar.SECOND, 0);
								c.set (Calendar.MILLISECOND, 0);
							}
							c.setTimeZone (TimeZone.getDefault ());
							return DateAccessor.formatISODateTimeWithTimezone (c);
						}
						LOGGER.info ("forecasted sub");
						Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
						c.setTimeZone (TimeZone.getDefault ());
						return DateAccessor.formatISODateTimeWithTimezone (c);

						// Calendar c = DateAccessor.parseISOTimeWithTimezone
						// (this.dailyTime);
						// c.add (Calendar.HOUR_OF_DAY, -this.hoursInPast);
						// // LOGGER.info ("ph " + this.hoursInPast);
						// if (this.hoursInPast == 24)
						// {
						// c.set (Calendar.HOUR_OF_DAY, 0);
						// c.set (Calendar.MINUTE, 0);
						// c.set (Calendar.SECOND, 0);
						// c.set (Calendar.MILLISECOND, 0);
						// }
						// // LOGGER.info ("start daily " +
						// // DateAccessor.formatISODateTimeWithTimezone (c));
						// // Underlying System works with local timezone!
						// c.setTimeZone (TimeZone.getDefault ());
						// // LOGGER.info ("start daily stz " +
						// // DateAccessor.formatISODateTimeWithTimezone (c));
						// return DateAccessor.formatISODateTimeWithTimezone (c);
					}
					catch (ParseException e)
					{
						e.printStackTrace ();
						throw PESCaDOExceptionEx.with ("Exception raised calculating start time", "", "FS", e);
					}
				}
			}
			// hourly
			else
			{
				if (timeType.equals (OBSERVED))
				{
					Calendar c = Calendar.getInstance ();
					// c.add (Calendar.HOUR_OF_DAY, -24);
					c.add (Calendar.HOUR_OF_DAY, -this.getHoursInPast ());
					if (this.hoursInPast == 24)
					{
						c.set (Calendar.HOUR_OF_DAY, 0);
						c.set (Calendar.MINUTE, 0);
						c.set (Calendar.SECOND, 0);
						c.set (Calendar.MILLISECOND, 0);
					}
					c.setTimeZone (TimeZone.getDefault ());
					return DateAccessor.formatISODateTimeWithTimezone (c);
				}
				Calendar c = Calendar.getInstance ();
				c.setTimeZone (TimeZone.getDefault ());
				return DateAccessor.formatISODateTimeWithTimezone (c);

				// Calendar c = Calendar.getInstance ();
				// c.add (Calendar.HOUR_OF_DAY, -this.hoursInPast);
				// if (this.hoursInPast == 24)
				// {
				// c.set (Calendar.HOUR_OF_DAY, 0);
				// c.set (Calendar.MINUTE, 0);
				// c.set (Calendar.SECOND, 0);
				// c.set (Calendar.MILLISECOND, 0);
				// }
				// c.setTimeZone (TimeZone.getDefault ());
				// // LOGGER.info ("start hourly " +
				// // DateAccessor.formatISODateTimeWithTimezone (c));
				// return DateAccessor.formatISODateTimeWithTimezone (c);
			}
		}
		// valid for detailed air quality report without Scheduler
		LOGGER.info ("useCase " + this.useCase + " USE-CASE " + USE_CASE);
		if ((this.submissionTime == null) && (!this.useCase.equals (USE_CASE)))
		{
			if (timeType.equals (OBSERVED))
			{
				LOGGER.info ("HUHU START OBSERVED SUB 24!!!!!!!!!!!!!!");
				Calendar c = Calendar.getInstance ();
				// c.add (Calendar.HOUR_OF_DAY, -24);
				c.add (Calendar.HOUR_OF_DAY, -this.getHoursInPast ());
				if (this.hoursInPast == 24)
				{
					c.set (Calendar.HOUR_OF_DAY, 0);
					c.set (Calendar.MINUTE, 0);
					c.set (Calendar.SECOND, 0);
					c.set (Calendar.MILLISECOND, 0);
				}
				c.setTimeZone (TimeZone.getDefault ());
				return DateAccessor.formatISODateTimeWithTimezone (c);
			}
			LOGGER.info ("HUHU START FORECASTED SUB!!!!!!!!!!!!!!");
			Calendar c = Calendar.getInstance ();
			c.setTimeZone (TimeZone.getDefault ());
			return DateAccessor.formatISODateTimeWithTimezone (c);
		}
		// valid for normal request
		return this.timeStart;
	}


	/**
	 * Set the start time.
	 * 
	 * @param start time
	 */
	public void setStartDateTime (String start)
	{
		this.timeStart = start;
	}


	/**
	 * Get the start date/time.
	 * 
	 * @return ISO encoded
	 * @throws PESCaDOException
	 */
	public String getEndDateTime () throws PESCaDOException
	{
		LOGGER.info ("timeEnd: " + this.timeEnd);
		// if (this.timeEnd == null)
		// {
		// vaid for Scheduler
		if (this.frequency != null)
		{
			if (this.frequency.equals ("daily"))
			{
				if (this.dailyTime != null)
				{
					try
					{
						Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
						c.add (Calendar.HOUR_OF_DAY, this.hoursInFuture);
						c.setTimeZone (TimeZone.getDefault ());
						// LOGGER.info ("end daily " +
						// DateAccessor.formatISODateTimeWithTimezone (c));
						return DateAccessor.formatISODateTimeWithTimezone (c);
					}
					catch (ParseException e)
					{
						e.printStackTrace ();
						throw PESCaDOExceptionEx.with ("Exception raised calculating end time", "", "FS", e);
					}
				}
			}
			// hourly
			else
			{
				Calendar c = Calendar.getInstance ();
				c.add (Calendar.HOUR_OF_DAY, this.hoursInFuture);
				c.setTimeZone (TimeZone.getDefault ());
				// LOGGER.info ("end hourly " +
				// DateAccessor.formatISODateTimeWithTimezone (c));
				return DateAccessor.formatISODateTimeWithTimezone (c);
			}
		}
		// valid for normal request
		else
		{
			// HACK for Iphone
			if (this.timeEnd.contains ("Z"))
			{
				LOGGER.info ("te " + this.timeEnd);
				String [] ta = this.timeEnd.split ("Z");
				String newTimeFormat = ta[0] + ":00+00:00";
				LOGGER.info ("ntf " + newTimeFormat);
				return newTimeFormat;
			}
			return this.timeEnd;
		}
		return this.timeEnd;
	}


	/**
	 * Get the start date/time.
	 * 
	 * @return ISO encoded
	 * @throws PESCaDOException
	 */
	public String getEndDateTime (String timeType) throws PESCaDOException
	{
		LOGGER.info ("timetype: " + timeType);
		LOGGER.info ("timeStart: " + this.timeStart);
		LOGGER.info ("timeEnd: " + this.timeEnd);
		LOGGER.info ("frequency: " + this.frequency);
		LOGGER.info ("dailyTime: " + this.dailyTime);
		LOGGER.info ("ph: " + this.getHoursInPast ());
		LOGGER.info ("fh: " + this.getHoursInFuture ());
		LOGGER.info ("subtime: " + this.submissionTime);
		LOGGER.info ("use case: " + this.useCase);
		// if (this.timeEnd == null)
		// {
		// vaid for Scheduler
		if (this.frequency != null)
		{
			if (this.frequency.equals ("daily"))
			{
				if (this.dailyTime != null)
				{
					try
					{
						if (timeType.equals (OBSERVED))
						{
							LOGGER.info ("observed sub");
							Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
							c.setTimeZone (TimeZone.getDefault ());
							return DateAccessor.formatISODateTimeWithTimezone (c);
						}
						LOGGER.info ("forecasted " + (this.getHoursInFuture ()));
						Calendar c = DateAccessor.parseISOTimeWithTimezone (this.dailyTime);
						// c.add (Calendar.HOUR_OF_DAY, 12);
						c.add (Calendar.HOUR_OF_DAY, this.getHoursInFuture ());
						c.setTimeZone (TimeZone.getDefault ());
						return DateAccessor.formatISODateTimeWithTimezone (c);

						// Calendar c = DateAccessor.parseISOTimeWithTimezone
						// (this.dailyTime);
						// c.add (Calendar.HOUR_OF_DAY, this.hoursInFuture);
						// c.setTimeZone (TimeZone.getDefault ());
						// // LOGGER.info ("end daily " +
						// // DateAccessor.formatISODateTimeWithTimezone (c));
						// return DateAccessor.formatISODateTimeWithTimezone (c);
					}
					catch (ParseException e)
					{
						e.printStackTrace ();
						throw PESCaDOExceptionEx.with ("Exception raised calculating end time", "", "FS", e);
					}
				}
			}
			// hourly
			else
			{
				if (timeType.equals (OBSERVED))
				{
					LOGGER.info ("HUHU END OBSERVED SUB !!!!!!!!!!!!!!");
					Calendar c = Calendar.getInstance ();
					c.setTimeZone (TimeZone.getDefault ());
					return DateAccessor.formatISODateTimeWithTimezone (c);
				}
				LOGGER.info ("HUHU END FORECASTED ADD 12!!!!!!!!!!!!!!");
				Calendar c = Calendar.getInstance ();
				// c.add (Calendar.HOUR_OF_DAY, 12);
				c.add (Calendar.HOUR_OF_DAY, this.getHoursInFuture ());
				c.setTimeZone (TimeZone.getDefault ());
				return DateAccessor.formatISODateTimeWithTimezone (c);
				// Calendar c = Calendar.getInstance ();
				// c.add (Calendar.HOUR_OF_DAY, this.hoursInFuture);
				// c.setTimeZone (TimeZone.getDefault ());
				// // LOGGER.info ("end hourly " +
				// // DateAccessor.formatISODateTimeWithTimezone (c));
				// return DateAccessor.formatISODateTimeWithTimezone (c);
			}
		}
		// valid for DetailedAirQualityReport without Scheduler
		LOGGER.info ("useCase " + this.useCase + " USE-CASE " + USE_CASE);
		if ((this.submissionTime == null) && (!this.useCase.equals (USE_CASE)))
		{
			if (timeType.equals (OBSERVED))
			{
				LOGGER.info ("HUHU END OBSERVED SUB !!!!!!!!!!!!!!");
				Calendar c = Calendar.getInstance ();
				c.setTimeZone (TimeZone.getDefault ());
				return DateAccessor.formatISODateTimeWithTimezone (c);
			}
			LOGGER.info ("HUHU END FORECASTED ADD 12!!!!!!!!!!!!!!");
			Calendar c = Calendar.getInstance ();
			// c.add (Calendar.HOUR_OF_DAY, 12);
			c.add (Calendar.HOUR_OF_DAY, this.getHoursInFuture ());
			c.setTimeZone (TimeZone.getDefault ());
			return DateAccessor.formatISODateTimeWithTimezone (c);
		}
		// }
		// valid for normal request
		return this.timeEnd;
	}


	/**
	 * Set the end time.
	 * 
	 * @param end time
	 */
	public void setEndDateTime (String end)
	{
		this.timeEnd = end;
	}


	/**
	 * Returns true if the geo type is an area.
	 * 
	 * @return True if the geo type is an area
	 */
	public boolean isArea ()
	{
		return this.isArea;
	}


	/**
	 * Get the polygon coordinates.
	 * 
	 * @return The polygon coordinates
	 */
	public List<Double> getPolygon ()
	{
		return this.polygon;
	}


	/**
	 * Get all time/date and space related parameters.
	 * 
	 * @return The parameters
	 * @throws PESCaDOException
	 */
	public List<services.pescado.common.ParameterType> getTimeSpaceParameters () throws PESCaDOException
	{
		final List<services.pescado.common.ParameterType> out = new ArrayList<services.pescado.common.ParameterType> (this.ps.size ());

		// geoType = route | area
		if (this.isArea)
			out.add (Parameters.createParameter (PARAM_GEO_AREA_TYPE, "http://www.pescado-project.eu/ontology/pescado.owl#regionGeoAreaType"));
		// TODO otherwise set the route

		if (this.polygon != null)
		{
			for (int i = 0; i < this.polygon.size (); i += 2)
			{
				final String point = new StringBuilder ().append (this.polygon.get (i)).append (',').append (this.polygon.get (i + 1)).toString ();
				out.add (Parameters.createParameter (PARAM_GEO_AREA_LOCATION, point));
			}
		}

		// timeStart, timeEnd = ISO 2011-02-16'T'13:02:10.2344
		out.add (Parameters.createParameter (PARAM_FROM_DATE_TIME, this.getStartDateTime ()));
		out.add (Parameters.createParameter (PARAM_TO_DATE_TIME, this.getEndDateTime ()));

		return out;
	}


	/**
	 * Get all space related parameters.
	 * 
	 * @return The parameters
	 */
	public List<services.pescado.common.ParameterType> getSpaceParameters ()
	{
		final List<services.pescado.common.ParameterType> out = new ArrayList<services.pescado.common.ParameterType> (this.ps.size ());

		// geoType = route | area
		if (this.isArea)
			out.add (Parameters.createParameter (PARAM_GEO_AREA_TYPE, "http://www.pescado-project.eu/ontology/pescado.owl#regionGeoAreaType"));
		// TODO otherwise set the route

		if (this.polygon != null)
		{
			for (int i = 0; i < this.polygon.size (); i += 2)
			{
				final String point = new StringBuilder ().append (this.polygon.get (i)).append (',').append (this.polygon.get (i + 1)).toString ();
				out.add (Parameters.createParameter (PARAM_GEO_AREA_LOCATION, point));
			}
		}

		return out;
	}


	/**
	 * Get the value of a parameter.
	 * 
	 * @param parameterName The name of the parameter
	 * @return The value or null if the parameter does not exist
	 */
	public List<Object> getValue (final String parameterName)
	{
		return this.ps.get (parameterName);
	}


	/**
	 * Get the value of a parameter as a text.
	 * 
	 * @param parameterName The name of the parameter
	 * @return The value as text or null if the parameter does not exist
	 */
	public final String getTextValue (final String parameterName)
	{
		final List<Object> value = this.getValue (parameterName);
		if (value == null)
			return null;
		final StringBuilder sb = new StringBuilder ();
		for (final Object o: value)
			sb.append (o.toString ());
		return sb.toString ();
	}


	/**
	 * Get the activity related parameters.
	 * 
	 * @return The activity related parameters
	 */
	public List<services.pescado.common.ParameterType> getActivityParameters ()
	{
		final List<services.pescado.common.ParameterType> out = new ArrayList<services.pescado.common.ParameterType> (this.ps.size ());
		for (final Entry<String, List<Object>> p: this.ps.entrySet ())
		{
			final String property = p.getKey ();
			if (!IGNORE_PARAMS.contains (property))
				out.add (Parameters.createParameter (NS_PESCADO + property, p.getValue ()));
		}
		return out;
	}


	/**
	 * Create a new parameter for PDGS.
	 * 
	 * @param property The property, an URI of the ontology
	 * @param value The value
	 * @return The parameter
	 */
	private static services.pescado.common.ParameterType createParameter (final String property, final String value)
	{
		return Parameters.createParameter (property, Collections.singleton ((Object) value));
	}


	/**
	 * Create a new parameter for PDGS.
	 * 
	 * @param property The property, an URI of the ontology
	 * @param value The value
	 * @return The parameter
	 */
	private static services.pescado.common.ParameterType createParameter (final String property, final Collection<? extends Object> value)
	{
		final services.pescado.common.ParameterType po = new services.pescado.common.ParameterType ();
		po.setProperty (property);
		po.getContent ().addAll (value);
		return po;
	}


	/**
	 * Return use case.
	 * 
	 * @return useCase
	 */
	public String getUseCase ()
	{
		return this.useCase;
	}


	/**
	 * Set use case.
	 * 
	 * @param useCase
	 */
	public void setUseCase (String useCase)
	{
		this.useCase = useCase;
	}


	/**
	 * Set past hours to consider.
	 * 
	 * @param ph
	 */
	public void setHoursInPast (int ph)
	{
		this.hoursInPast = ph;
	}


	/**
	 * Set future hours to consider.
	 * 
	 * @param fh
	 */
	public void setHoursInFuture (int fh)
	{
		this.hoursInFuture = fh;
	}

}
