package services.pescado.fusionservice.impl;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.opengis.om._1.ObservationCollectionType;

import org.junit.Assert;
import org.junit.Test;

import services.pescado.common.PESCaDOException;
import services.pescado.common.ParameterType;
import de.fraunhofer.iosb.sosclient.OWSException;


/**
 * Tests DRS retrieval and storage to KBS.
 * 
 * @author J&uuml;rgen Mo&szlig;graber
 */
public class TestDRS
{
	protected static final Pattern	DATETIMEPATTERN	= Pattern.compile ("(-?[0-9]{4,})" + "-" + "([0-9]{2})" + "-" + "([0-9]{2})" + "T" + "([0-9]{2})" + ":" + "([0-9]{2})" + ":" + "([0-9]{2})([.]([0-9]{1,3}))?" + "((Z)|(([+]|-)([0-9]{2}):([0-9]{2})))?");


	/**
	 * Tests a date/time parsing problem with HermiT.
	 */

	@Test
	public void testDateTimeParsing ()
	{
		System.out.println (DATETIMEPATTERN);

		Matcher matcher = DATETIMEPATTERN.matcher ("2011-04-04T00:00:00");
		Assert.assertTrue (matcher.matches ());

		matcher = DATETIMEPATTERN.matcher ("2011-04-04T00:00:00+02:00");
		Assert.assertTrue (matcher.matches ());

		Assert.assertNotNull (FusionServiceImpl.incTime ("2011-04-04T01:12:00.000+02:00"));
	}


	/**
	 * A demo SOS request.
	 * 
	 * @throws IOException Could not retrieve the data
	 * @throws ParseException Could not parse result values
	 * @throws OWSException The SOS server returned an exception
	 * @throws PESCaDOException
	 */

	@Test
	public void testDataRetrievalAndStorage () throws IOException, ParseException, OWSException, PESCaDOException
	{
		// final FusionServiceImpl as = new FusionServiceImpl (null, null);
		final FusionServiceImpl as = new FusionServiceImpl ();

		final List<ParameterType> parameter = new ArrayList<ParameterType> ();
		final ParameterType p1 = new ParameterType ();
		p1.setProperty ("timeStart");
		p1.getContent ().add ("2011-04-05T00:00:00+02:00");
		parameter.add (p1);
		final ParameterType p2 = new ParameterType ();
		p2.setProperty ("timeEnd");
		p2.getContent ().add ("2011-04-06T00:00:00+02:00");
		parameter.add (p2);
		final ParameterType p3 = new ParameterType ();
		p3.setProperty ("geoType");
		p3.getContent ().add ("area");
		parameter.add (p3);
		final ParameterType p4 = new ParameterType ();
		p4.setProperty ("geoPolygon");
		p4.getContent ().add ("60.46060328697707,24.92523193359375 60.46060328697707,24.171295166015625 60.13740224199459,24.171295166015625 60.13740224199459,24.92523193359375");
		parameter.add (p4);

		final Parameters params = new Parameters (parameter);
		// Old version of input param for retrieveData
		final Map<String, Collection<String>> relevantAspects = new HashMap<String, Collection<String>> ();
		final Collection<String> properties = new ArrayList<String> ();
		properties.add ("http://www.pescado-project.eu/ontology/pescado.owl#O3");
		properties.add ("http://www.pescado-project.eu/ontology/pescado.owl#NO2");
		relevantAspects.put ("http://www.pescado-project.eu/ontology/pescado.owl#AirQualityDataType", properties);
		// Old version of input param for retrieveData end

		// New version of input param for retrieveData
		final List<ParameterType> racsParameters = new ArrayList<services.pescado.common.ParameterType> ();
		final Set<Entry<String, Collection<String>>> entrySet = relevantAspects.entrySet ();
		final Iterator<Entry<String, Collection<String>>> entrySetIterator = entrySet.iterator ();
		while (entrySetIterator.hasNext ())
		{
			final Entry<String, Collection<String>> racsValues = entrySetIterator.next ();
			final ParameterType racsParameter = new ParameterType ();
			final String key = racsValues.getKey ();
			racsParameter.setProperty (key);
			final Collection<String> racsValuesV = racsValues.getValue ();
			final Iterator<String> racsValuesIterator = racsValuesV.iterator ();
			String combinedRacsValue = "";
			while (racsValuesIterator.hasNext ())
			{
				final String singleRacsValue = racsValuesIterator.next ();
				combinedRacsValue = combinedRacsValue.concat (singleRacsValue);
				combinedRacsValue = combinedRacsValue.concat (" ");
			}
			racsParameter.getContent ().add (combinedRacsValue);
			racsParameters.add (racsParameter);
		}
		// New version of input param for retrieveData end

		// final Map<String, ObservationCollectionType> data = as.retrieveData
		// (params, relevantAspects);
		final String environmentalDataNature = "http://www.pescado-project.eu/ontology/pescado.owl#forecasted";
		final Map<String, ObservationCollectionType> data = as.retrieveData (params, racsParameters, environmentalDataNature);
		Assert.assertNotNull (data);

		String [][] parsed_data = as.parse_retrieveData (data);
		System.out.println ("Result: ");
		for (int i = 0; i < parsed_data.length; i++)
		{
			String [] pElement = parsed_data[i];
			for (int j = 0; j < pElement.length; j++)
			{
				System.out.println (pElement[j]);
			}
		}

	}

}
